@@ -5,10 +5,11 @@ mod field;
 mod mul;
 pub(crate) mod projective;
 pub(crate) mod scalar;
-mod util;
+
+#[cfg(test)]
+mod dev;
 
 pub use field::FieldElement;
-pub use mul::lincomb;
 
 use affine::AffinePoint;
 use projective::ProjectivePoint;
@@ -38,13 +39,12 @@ mod tests {
     }
 
     #[test]
-    #[cfg(feature = "zeroize")]
     fn generate_secret_key() {
         use crate::SecretKey;
         use elliptic_curve::rand_core::OsRng;
         let key = SecretKey::random(&mut OsRng);
 
         // Sanity check
-        assert!(!key.to_bytes().iter().all(|b| *b == 0))
+        assert!(!key.to_be_bytes().iter().all(|b| *b == 0))
     }
 }
