@@ -65,9 +65,16 @@
 //! In experiments, I was not able to detect any case where they would go outside the 128 bit bound,
 //! but I cannot be sure that it cannot happen.
 
-use crate::arithmetic::{scalar::Scalar, ProjectivePoint};
+use crate::arithmetic::{
+    scalar::{Scalar, WideScalar},
+    ProjectivePoint,
+};
 use core::ops::{Mul, MulAssign};
-use elliptic_curve::subtle::{Choice, ConditionallySelectable, ConstantTimeEq};
+use elliptic_curve::{
+    ops::LinearCombination,
+    subtle::{Choice, ConditionallySelectable, ConstantTimeEq},
+    IsHigh,
+};
 
 /// Lookup table containing precomputed values `[p, 2p, 3p, ..., 8p]`
 #[derive(Copy, Clone, Default)]
@@ -136,12 +143,9 @@ const G2: Scalar = Scalar::from_bytes_unchecked(&[
 
 /// Find r1 and r2 given k, such that r1 + r2 * lambda == k mod n.
 fn decompose_scalar(k: &Scalar) -> (Scalar, Scalar) {
-    // these _var calls are constant time since the shift amount is constant
-    let c1 = k.mul_shift_var(&G1, 272);
-    let c2 = k.mul_shift_var(&G2, 272);
-
-    let c1 = c1 * MINUS_B1;
-    let c2 = c2 * MINUS_B2;
+    // these _vartime calls are constant time since the shift amount is constant
+    let c1 = WideScalar::mul_shift_vartime(k, &G1, 272) * MINUS_B1;
+    let c2 = WideScalar::mul_shift_vartime(k, &G2, 272) * MINUS_B2;
     let r2 = c1 + c2;
     let r1 = k + r2 * MINUS_LAMBDA;
 
@@ -252,7 +256,7 @@ fn lincomb_generic<const N: usize>(xs: &[ProjectivePoint; N], ks: &[Scalar; N])
 
     let tables1 = static_zip_map(
         |x, r_sign| LookupTable::from(&ProjectivePoint::conditional_select(&x, &-x, r_sign)),
-        &xs,
+        xs,
         &r1_signs,
         LookupTable::default(),
     );
@@ -298,14 +302,15 @@ fn mul(x: &ProjectivePoint, k: &Scalar) -> ProjectivePoint {
     lincomb_generic(&[*x], &[*k])
 }
 
-/// Calculates `x * k + y * l`.
-pub fn lincomb(
-    x: &ProjectivePoint,
-    k: &Scalar,
-    y: &ProjectivePoint,
-    l: &Scalar,
-) -> ProjectivePoint {
-    lincomb_generic(&[*x, *y], &[*k, *l])
+impl LinearCombination for ProjectivePoint {
+    fn lincomb(
+        x: &ProjectivePoint,
+        k: &Scalar,
+        y: &ProjectivePoint,
+        l: &Scalar,
+    ) -> ProjectivePoint {
+        lincomb_generic(&[*x, *y], &[*k, *l])
+    }
 }
 
 impl Mul<Scalar> for ProjectivePoint {
@@ -346,10 +351,8 @@ impl MulAssign<&Scalar> for ProjectivePoint {
 
 #[cfg(test)]
 mod tests {
-    use super::lincomb;
     use crate::arithmetic::{ProjectivePoint, Scalar};
-    use elliptic_curve::rand_core::OsRng;
-    use elliptic_curve::{Field, Group};
+    use elliptic_curve::{ops::LinearCombination, rand_core::OsRng, Field, Group};
 
     #[test]
     fn test_lincomb() {
@@ -359,7 +362,7 @@ mod tests {
         let l = Scalar::random(&mut OsRng);
 
         let reference = &x * &k + &y * &l;
-        let test = lincomb(&x, &k, &y, &l);
+        let test = ProjectivePoint::lincomb(&x, &k, &y, &l);
         assert_eq!(reference, test);
     }
 }
