@@ -1,4 +1,4 @@
-//! ECDSA signer
+//! ECDSA signing support.
 
 use super::{recoverable, Error, Signature, VerifyingKey};
 use crate::{FieldBytes, NonZeroScalar, ProjectivePoint, PublicKey, Scalar, Secp256k1, SecretKey};
@@ -7,25 +7,27 @@ use core::{
     fmt::{self, Debug},
 };
 use ecdsa_core::{
-    hazmat::{FromDigest, RecoverableSignPrimitive},
-    rfc6979,
+    hazmat::{rfc6979_generate_k, SignPrimitive},
     signature::{
         digest::{BlockInput, FixedOutput, Reset, Update},
         DigestSigner, RandomizedDigestSigner,
     },
 };
 use elliptic_curve::{
+    bigint::U256,
     consts::U32,
-    ops::Invert,
+    ops::{Invert, Reduce},
     rand_core::{CryptoRng, RngCore},
     subtle::{Choice, ConstantTimeEq},
+    zeroize::Zeroize,
+    IsHigh,
 };
 
 #[cfg(any(feature = "keccak256", feature = "sha256"))]
 use ecdsa_core::signature::{self, digest::Digest, PrehashSignature, RandomizedSigner};
 
 #[cfg(feature = "pkcs8")]
-use crate::pkcs8::{self, FromPrivateKey};
+use crate::pkcs8::{self, DecodePrivateKey};
 
 #[cfg(feature = "pem")]
 use core::str::FromStr;
@@ -48,8 +50,8 @@ impl SigningKey {
 
     /// Initialize [`SigningKey`] from a raw scalar value (big endian).
     pub fn from_bytes(bytes: &[u8]) -> Result<Self, Error> {
-        let inner = SecretKey::from_bytes(bytes)
-            .map(|sk| sk.to_secret_scalar())
+        let inner = SecretKey::from_be_bytes(bytes)
+            .map(|sk| sk.to_nonzero_scalar())
             .map_err(|_| Error::new())?;
 
         Ok(Self { inner })
@@ -62,12 +64,6 @@ impl SigningKey {
         }
     }
 
-    /// Legacy alias for [`SigningKey::verifying_key`].
-    #[deprecated(since = "0.9.3", note = "use `verifying_key()` instead")]
-    pub fn verify_key(&self) -> VerifyingKey {
-        self.verifying_key()
-    }
-
     /// Serialize this [`SigningKey`] as bytes
     pub fn to_bytes(&self) -> FieldBytes {
         self.inner.to_bytes()
@@ -110,14 +106,12 @@ impl<D> DigestSigner<D, recoverable::Signature> for SigningKey
 where
     D: BlockInput + FixedOutput<OutputSize = U32> + Clone + Default + Reset + Update,
 {
-    fn try_sign_digest(&self, digest: D) -> Result<recoverable::Signature, Error> {
-        let ephemeral_scalar = rfc6979::generate_k(&self.inner, digest.clone(), &[]);
-        let msg_scalar = Scalar::from_digest(digest);
-        let (signature, recovery_id) = self
-            .inner
-            .try_sign_recoverable_prehashed(ephemeral_scalar.as_ref(), &msg_scalar)?;
-
-        recoverable::Signature::new(&signature, recoverable::Id(recovery_id as u8))
+    fn try_sign_digest(&self, msg_digest: D) -> Result<recoverable::Signature, Error> {
+        let z = <Scalar as Reduce<U256>>::from_be_bytes_reduced(msg_digest.finalize_fixed());
+        let k = rfc6979_generate_k::<_, D>(&self.inner, &z, &[]);
+        let (signature, recid) = self.inner.try_sign_prehashed(**k, z)?;
+        let recoverable_id = recid.ok_or_else(Error::new)?.try_into()?;
+        recoverable::Signature::new(&signature, recoverable_id)
     }
 }
 
@@ -142,28 +136,26 @@ where
     fn try_sign_digest_with_rng(
         &self,
         mut rng: impl CryptoRng + RngCore,
-        digest: D,
+        msg_digest: D,
     ) -> Result<recoverable::Signature, Error> {
         let mut added_entropy = FieldBytes::default();
         rng.fill_bytes(&mut added_entropy);
 
-        let ephemeral_scalar = rfc6979::generate_k(&self.inner, digest.clone(), &added_entropy);
-        let msg_scalar = Scalar::from_digest(digest);
-        let (signature, is_r_odd) = self
-            .inner
-            .try_sign_recoverable_prehashed(ephemeral_scalar.as_ref(), &msg_scalar)?;
-
-        recoverable::Signature::new(&signature, recoverable::Id(is_r_odd as u8))
+        let z = <Scalar as Reduce<U256>>::from_be_bytes_reduced(msg_digest.finalize_fixed());
+        let k = rfc6979_generate_k::<_, D>(&self.inner, &z, &added_entropy);
+        let (signature, recid) = self.inner.try_sign_prehashed(**k, z)?;
+        let recoverable_id = recid.ok_or_else(Error::new)?.try_into()?;
+        recoverable::Signature::new(&signature, recoverable_id)
     }
 }
 
-impl RecoverableSignPrimitive<Secp256k1> for Scalar {
+impl SignPrimitive<Secp256k1> for Scalar {
     #[allow(non_snake_case, clippy::many_single_char_names)]
-    fn try_sign_recoverable_prehashed<K>(
+    fn try_sign_prehashed<K>(
         &self,
-        ephemeral_scalar: &K,
-        z: &Scalar,
-    ) -> Result<(Signature, bool), Error>
+        ephemeral_scalar: K,
+        z: Scalar,
+    ) -> Result<(Signature, Option<ecdsa_core::RecoveryId>), Error>
     where
         K: Borrow<Scalar> + Invert<Output = Scalar>,
     {
@@ -181,7 +173,7 @@ impl RecoverableSignPrimitive<Secp256k1> for Scalar {
 
         // Lift x-coordinate of 𝐑 (element of base field) into a serialized big
         // integer, then reduce it into an element of the scalar field
-        let r = Scalar::from_bytes_reduced(&R.x.to_bytes());
+        let r = <Scalar as Reduce<U256>>::from_be_bytes_reduced(R.x.to_bytes());
 
         // Compute `s` as a signature over `r` and `z`.
         let s = k_inverse * (z + (r * self));
@@ -190,10 +182,13 @@ impl RecoverableSignPrimitive<Secp256k1> for Scalar {
             return Err(Error::new());
         }
 
-        let mut signature = Signature::from_scalars(r, s)?;
-        let is_r_odd = bool::from(R.y.normalize().is_odd());
-        let is_s_high = signature.normalize_s()?;
-        Ok((signature, is_r_odd ^ is_s_high))
+        let signature = Signature::from_scalars(r, s)?;
+        let is_r_odd: bool = R.y.normalize().is_odd().into();
+        let is_s_high: bool = signature.s().is_high().into();
+        let signature_low = signature.normalize_s().unwrap_or(signature);
+        let recovery_id = ecdsa_core::RecoveryId::new(is_r_odd ^ is_s_high, false);
+
+        Ok((signature_low, Some(recovery_id)))
     }
 }
 
@@ -227,7 +222,7 @@ impl From<SecretKey> for SigningKey {
 impl From<&SecretKey> for SigningKey {
     fn from(secret_key: &SecretKey) -> SigningKey {
         Self {
-            inner: secret_key.to_secret_scalar(),
+            inner: secret_key.to_nonzero_scalar(),
         }
     }
 }
@@ -272,16 +267,26 @@ impl From<&NonZeroScalar> for SigningKey {
     }
 }
 
-#[cfg(feature = "pkcs8")]
-#[cfg_attr(docsrs, doc(cfg(feature = "pkcs8")))]
-impl FromPrivateKey for SigningKey {
-    fn from_pkcs8_private_key_info(
-        private_key_info: pkcs8::PrivateKeyInfo<'_>,
-    ) -> pkcs8::Result<Self> {
-        SecretKey::from_pkcs8_private_key_info(private_key_info).map(Into::into)
+impl Drop for SigningKey {
+    fn drop(&mut self) {
+        self.inner.zeroize();
     }
 }
 
+#[cfg(feature = "pkcs8")]
+#[cfg_attr(docsrs, doc(cfg(feature = "pkcs8")))]
+impl TryFrom<pkcs8::PrivateKeyInfo<'_>> for SigningKey {
+    type Error = pkcs8::Error;
+
+    fn try_from(private_key_info: pkcs8::PrivateKeyInfo<'_>) -> pkcs8::Result<Self> {
+        SecretKey::try_from(private_key_info).map(Into::into)
+    }
+}
+
+#[cfg(feature = "pkcs8")]
+#[cfg_attr(docsrs, doc(cfg(feature = "pkcs8")))]
+impl DecodePrivateKey for SigningKey {}
+
 #[cfg(feature = "pem")]
 #[cfg_attr(docsrs, doc(cfg(feature = "pem")))]
 impl FromStr for SigningKey {
